#ifndef POINT_H
#define POINT_H

#include <initializer_list>
#include <ostream>

using namespace std;

template<size_t N = 3, typename T = double>
class Point
{
    T array[N];
public:
    Point() = default;
    Point(initializer_list<T> values) {
        for (size_t i {0}; i != values.size() && i != N; ++i)
            array[i] = values.begin()[i];
        for (size_t i {values.size()}; i != N; ++i)
            array[i] = 0;
    }
    friend ostream& operator<<(ostream& os, const Point<N,T>& point) {
        os << "[";
        for (size_t i {0}; i != N; ++i) {
            os << point.array[i];
            if (i != N - 1)
                os << ", ";
        }
        os << "]";
        return os;
    }
    Point operator-() const {
        Point p;
        for (size_t i {0}; i != N; ++i)
            p.array[i] = -array[i];
        return p;
    }
    Point operator+=(const Point& other) {
        for (size_t i {0}; i != N; ++i)
            array[i] += other.array[i];
        return *this;
    }
    Point& operator-=(const Point& other) {
        *this+=(-other);
        return *this;
    }
    bool operator==(const Point& other) {
        if (this == &other)
            return true;
        for (size_t i {0}; i != N; ++i)
            if (array[i] != other.array[i])
                return false;
        return true;
    }
    bool operator!=(const Point& other) {
        return !(*this == other);
    }
    friend Point operator*(double s, const Point& point) {
        Point q;
        for (size_t i {0}; i != N; ++i)
            q.array[i] = s*point.array[i];
        return q;
    }
    friend T operator*(const Point& a, const Point& b) {
        T res {0};
        for (size_t i {0}; i != N; ++i)
            res += a.array[i]*b.array[i];
        return res;
    }
    T& operator[](size_t i) {
        return array[i];
    }
    const T& operator[](size_t i) const {
        return array[i];
    }
};

template<size_t N, typename T>
Point<N, T> operator+(const Point<N, T>& a, const Point<N, T>& b) {
    Point<N, T> p {a};
    p += b;
    return p;
}

template<size_t N, typename T>
Point<N, T> operator-(const Point<N, T>& a, const Point<N, T>& b) {
    return a + (-b);
}

using Point3D = Point<>;
using Point2DInt = Point<2, int>;

#endif // POINT_H
